% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions.R
\name{STITCH}
\alias{STITCH}
\title{Sequencing To Imputation Through Constructing Haplotypes}
\usage{
STITCH(chr, nGen = "", posfile, K, outputdir, tempdir, bamlist = "",
  cramlist = "", reference = "", genfile = "", method = "diploid",
  outputInputInVCFFormat = FALSE, downsampleToCov = 50,
  downsampleFraction = 1, readAware = TRUE, chrStart = NA, chrEnd = NA,
  regionStart = NA, regionEnd = NA, buffer = NA,
  maxDifferenceBetweenReads = 1000, alphaMatThreshold = 1e-04,
  emissionThreshold = 1e-04, iSizeUpperLimit = as.integer(600),
  bqFilter = as.integer(17), niterations = 40,
  shuffleHaplotypeIterations = c(4, 8, 12, 16), splitReadIterations = 25,
  nCores = 1, expRate = 0.5, maxRate = 100, minRate = 0.1, Jmax = 100,
  regenerateInput = TRUE, originalRegionName = NA,
  keepInterimFiles = FALSE, keepTempDir = FALSE, environment = "server",
  pseudoHaploidModel = 9, diploidModel = 2,
  outputHaplotypeProbabilities = FALSE, switchModelIteration = NA,
  generateInputOnly = FALSE, restartIterations = NA,
  refillIterations = c(6, 10, 14, 18), downsampleSamples = 1,
  downsampleSamplesKeepList = NA, subsetSNPsfile = NA,
  useSoftClippedBases = FALSE, outputBlockSize = 1000)
}
\arguments{
\item{chr}{What chromosome to run. Should match BAM headers}

\item{posfile}{Where to find file with positions to run. File is tab seperated with no header, one row per SNP, with col 1 = chromosome, col 2 = physical position (sorted from smallest to largest), col 3 = reference base, col 4 = alternate base. Bases are capitalized. Example first row: 1<tab>1000<tab>A<tab>G<tab>}

\item{K}{How many founder / mosaic haplotypes to use}

\item{outputdir}{What output directory to use}

\item{tempdir}{What directory to use as temporary directory. If possible, use ramdisk, like /dev/shm/}

\item{bamlist}{Path to file with bam file locations. File is one row per entry, path to bam files. Bam index files should exist in same directory as for each bam, suffixed either .bam.bai or .bai}

\item{cramlist}{Path to file with cram file locations. File is one row per entry, path to cram files. cram files are converted to bam files on the fly for parsing into STITCH}

\item{reference}{Path to reference fasta used for making cram files. Only required if cramlist is defined}

\item{genfile}{Path to gen file with high coverage results. Empty for no genfile. File has a header row with a name for each sample, matching what is found in the bam file. Each subject is then a tab seperated column, with 0 = hom ref, 1 = het, 2 = hom alt and NA indicating missing genotype, with rows corresponding to rows of the posfile. Note therefore this file has one more row than posfile which has no header}

\item{method}{How to run imputation - either diploid or pseudoHaploid, the former being the original method quadratic in K, the later being linear in K}

\item{outputInputInVCFFormat}{Whether to output the input in vcf format}

\item{downsampleToCov}{What coverage to downsample individual sites to. This ensures no floating point errors at sites with really high coverage}

\item{downsampleFraction}{Downsample BAMs by choosing a fraction of reads to retain. Must be value 0<downsampleFraction<1}

\item{readAware}{Whether to run the algorithm is read aware mode. If false, then reads are split into new reads, one per SNP per read}

\item{chrStart}{When loading from BAM, some start position, before SNPs occur. Default NA will infer this from either regionStart, regionEnd and buffer, or posfile}

\item{chrEnd}{When loading from BAM, some end position, after SNPs occur. Default NA will infer this from either regionStart, regionEnd and buffer, or posfile}

\item{regionStart}{When running imputation, where to start from. The 1-based position x is kept if regionStart <= x <= regionEnd}

\item{regionEnd}{When running imputation, where to stop}

\item{buffer}{Buffer of region to perform imputation over. So imputation is run form regionStart-buffer to regionEnd+buffer, and reported for regionStart to regionEnd, including the bases of regionStart and regionEnd}

\item{maxDifferenceBetweenReads}{How much of a difference to allow the reads to make in the forward backward probability calculation. For example, if P(read | state 1)=1 and P(read | state 2)=1e-6, re-scale so that their ratio is this value. This helps prevent any individual read as having too much of an influence on state changes, helping prevent against influence by false positive SNPs}

\item{alphaMatThreshold}{Minimum (maximum is 1 minus this) state switching into probabilities}

\item{emissionThreshold}{Emission probability bounds. emissionThreshold < P(alt read | state k) < (1-emissionThreshold)}

\item{iSizeUpperLimit}{Do not use reads with an insert size of more than this value}

\item{bqFilter}{Minimum BQ for a SNP in a read. Also, the algorithm uses bq<=mq, so if mapping quality is less than this, the read isnt used}

\item{niterations}{Number of EM iterations}

\item{shuffleHaplotypeIterations}{Iterations on which to perform heuristic attempt to shuffle founder haplotypes for better fit. To disable set to NA.}

\item{splitReadIterations}{Iterations to try and split reads which may span recombination breakpoints for a better fit}

\item{nCores}{How many cores to use}

\item{expRate}{Expected recombination rate in cM/Mb}

\item{maxRate}{Maximum recomb rate cM/Mb}

\item{minRate}{Minimum recomb rate cM/Mb}

\item{Jmax}{Maximum number of SNPs on a read}

\item{regenerateInput}{Whether to regenerate input files}

\item{originalRegionName}{If regenerateInput is FALSE (i.e. using existing data), this is the name of the original region name (chr.regionStart.regionEnd). This is necessary to load past variables}

\item{keepInterimFiles}{Whether to keep interim parameter estimates}

\item{keepTempDir}{Whether to keep files in temporary directory}

\item{environment}{Whether to use server or cluster multicore options}

\item{pseudoHaploidModel}{How to model read probabilities in pseudo diploid model (shouldn't be changed)}

\item{diploidModel}{How to model read probabilities in diploid model (shouldn't be changed)}

\item{outputHaplotypeProbabilities}{Whether to output haplotype probabilities in files}

\item{switchModelIteration}{Whether to switch from pseudoHaploid to diploid and at what iteration (NA for no switching)}

\item{generateInputOnly}{Whether to just generate input data then quit}

\item{restartIterations}{In pseudoHaploid method, which iterations to look for collapsed haplotype prnobabilities to resolve}

\item{refillIterations}{When to try and refill some of the less frequently used haplotypes}

\item{downsampleSamples}{What fraction of samples to retain. Useful for checking effect of N on imputation. Not meant for general use}

\item{downsampleSamplesKeepList}{When downsampling samples, specify a numeric list of samples to keep}

\item{subsetSNPsfile}{If input data has already been made for a region, then subset down to a new set of SNPs, as given by this file. Not meant for general use}

\item{useSoftClippedBases}{Whether to use (TRUE) or not use (FALSE) bases in soft clipped portions of reads}

\item{outputBlockSize}{How many samples to write out to disk at the same time when making temporary VCFs that are later pasted together at the end to make the final VCF. Smaller means lower RAM footprint, larger means faster write.}

\item{Number}{of generations since founding or mixing. Note that the algorithm is relatively robust to this. Use nGen = 4 * Ne / K if unsure}
}
\value{
Results in properly formatted version
}
\author{
Robert Davies
}

